/*
 * Translatable strings file generated by Glade.
 * Add this file to your project's POTFILES.in.
 * DO NOT compile it as part of your application.
 */

gchar *s = N_("Straw Preferences");
gchar *s = N_("<b>Global</b>");
gchar *s = N_("_Refresh frequency:");
gchar *s = N_("_Number of articles per feed:");
gchar *s = N_("_Display location of new items:");
gchar *s = N_("_top of the list");
gchar *s = N_("_bottom of the list");
gchar *s = N_("<b>Web Browser</b>");
gchar *s = N_("_Override preferred browser setting:");
gchar *s = N_(" ");
gchar *s = N_("<small><i>(example: <b>epiphany %s</b>)</i></small>");
gchar *s = N_("_General Settings");
gchar *s = N_("<span weight=\"bold\">C_ategories\n"
              "</span>");
gchar *s = N_("    ");
gchar *s = N_("Location:");
gchar *s = N_("User name:");
gchar *s = N_("Password:");
gchar *s = N_("Refresh frequency:");
gchar *s = N_("Use _Default");
gchar *s = N_("<span weight=\"bold\">_External Source for Category Contents:</span>");
gchar *s = N_("<span weight=\"bold\">_Sort Category Contents:</span>");
gchar *s = N_("A_scending");
gchar *s = N_("_Categories");
gchar *s = N_("Feed Properties");
gchar *s = N_("<b>Source Information</b>");
gchar *s = N_("    ");
gchar *s = N_("Feed Description");
gchar *s = N_("Feed Copyright");
gchar *s = N_("<b>Refresh times</b>");
gchar *s = N_("  ");
gchar *s = N_("Previous Refresh:");
gchar *s = N_("Next Refresh:");
gchar *s = N_("_Information");
gchar *s = N_("<b>Settings</b>");
gchar *s = N_("  ");
gchar *s = N_("_Title:");
gchar *s = N_("_Location:");
gchar *s = N_("<b>Polling and Article Storage</b>");
gchar *s = N_("  ");
gchar *s = N_("_Refresh frequency (minutes):");
gchar *s = N_("_Number of articles stored:");
gchar *s = N_("Us_e Default");
gchar *s = N_("Use _Default");
gchar *s = N_("<b>Categories</b>");
gchar *s = N_("  ");
gchar *s = N_("Select the categories to which this feed should belong to.");
gchar *s = N_("_Settings");
gchar *s = N_("<b>Authorization</b>");
gchar *s = N_("  ");
gchar *s = N_("Use this only if you need authorization to read this feed.");
gchar *s = N_("_Username:");
gchar *s = N_("_Password:");
gchar *s = N_("_Authorization");
gchar *s = N_("Subscribe");
gchar *s = N_("<b><big>Subscribe to Feed</big></b>");
gchar *s = N_("Feed URL:");
gchar *s = N_("_Continue");
gchar *s = N_("Main");
gchar *s = N_("Connecting to ...");
gchar *s = N_("Progress");
gchar *s = N_("<big><b>Authorization Required</b></big>");
gchar *s = N_("The site you're accessing requires authorization.");
gchar *s = N_("_Username:");
gchar *s = N_("_Password:");
gchar *s = N_("_OK");
gchar *s = N_("Auth");
gchar *s = N_("Select the feeds you want to subscribe to below:");
gchar *s = N_("_Done");
gchar *s = N_("Multiple Feed");
gchar *s = N_("Straw");
gchar *s = N_("_Subscribe to New Feed");
gchar *s = N_("_Import Subscriptions");
gchar *s = N_("_Export Subscriptions");
gchar *s = N_("Mark Feed as _Read");
gchar *s = N_("Mark _All Feeds As Read");
gchar *s = N_("Re_move Selected Feed");
gchar *s = N_("Subscription _Information");
gchar *s = N_("Refresh _Selected");
gchar *s = N_("Refresh _Category");
gchar *s = N_("Refresh _All");
gchar *s = N_("_Go");
gchar *s = N_("_Previous Item");
gchar *s = N_("_Next Item");
gchar *s = N_("Scroll or Next _Unread Item");
gchar *s = N_("P_revious Feed");
gchar *s = N_("N_ext Feed");
gchar *s = N_("Ne_xt Feed with Unread");
gchar *s = N_("Previous Ca_tegory");
gchar *s = N_("Next _Category");
gchar *s = N_("Report a _problem");
gchar *s = N_("Refresh all feeds");
gchar *s = N_("Refresh");
gchar *s = N_("Scroll to the next unread article");
gchar *s = N_("Scroll Unread");
gchar *s = N_("Add a new feed to the subscription list");
gchar *s = N_("Subscribe");
gchar *s = N_("_Subscriptions:");
gchar *s = N_("label216");
gchar *s = N_("_Search items for:");
gchar *s = N_("_Date Filter");
gchar *s = N_("Earliest date");
gchar *s = N_("Latest date");
gchar *s = N_("Return to feed list");
gchar *s = N_("label217");
gchar *s = N_("_Next Refresh:");
gchar *s = N_("0 items found");
