body {
	margin: 0.5em 0.5em 0.5em 0.5em;
}

img {  padding: 0.8em; }


/* article head */
#itemhead {
         margin-bottom:10px;
         width:100%;
}

#itemheader {
  /* the following are replaced by straw to reflect the gtk theme */
/*  /***HEADERBG*/
  /***HEADERFG*/
  /***HEADERBORDERCOLOR*/ */
}
 

td.headleft {
         padding: 2px 2px 2px 8px;
         text-align:right;
         font-weight: bold;
         white-space: nowrap;
}

td#date {
    text-align: right;
    font-weight: normal;
    white-space: nowrap;
    padding: 2px 2px 2px 8px;
}

td.headright {
         padding:2px 5px;
         width: 100%;
}

/* "Main" text */
div.description {
	margin: 15px 15px 10px 15px;
	text-align: justify;
}

/* Dublin Core Information */
div.dcinfo {
}
	
table.dc {
	width: 100%;
	border-width:1px;
	border-top-style:solid;
}

tr.dc {
	border-width:0;
	border-bottom-width:1px;
	border-style:dashed;
}

td.dcname {
	padding: 1px 0.5em 1px 0.5em;
	width: 30%;
        font-weight: bold;
}

td.dcvalue {
	padding: 1px 0.5em 1px 0.5em;
}

a.dclink {
}

p.emptyfeed {
	font-size: 1.4em;
	font-weight: bold;
}

blockquote { 
        border-width: 1px;
        border-style: solid;
        padding: 2em;
        margin: 2em;

  /***BQUOTEBORDERCOLOR*/
  /***BQUOTEFG*/
  /***BQUOTEBG*/

}

blockquote > blockquote { 
        border: 0px;
}

.title {
    font-size: 14pt;
    padding: 4px;
}
